/*
 * @Author: your name
 * @Date: 2021-09-18 09:51:34
 * @LastEditTime: 2022-03-09 20:55:27
 * @LastEditors: Please set LastEditors
 * @Description: In User Settings Edit
 * @FilePath: /NOP_EHR/media/igs/xnm/ros/RM/roadmodel/modules/roadmodel.h
 */

#pragma once

#include <memory>
#include <string>
#include <vector>

#include "delay.h"
#include "dependency_injector.h"
#include "frame.h"
#include "local_view.h"
// #include "mpc_drv_msgs/msg/rmsm_message.hpp"
#include "dpc/impl_type_rmsmmessage.h"
// #include "mpc_drv_msgs/msg/road_info.hpp"
#include "dpc/impl_type_roadinfo.h"
// #include "mpc_drv_msgs/msg/targets_info.hpp"
#include "dpc/impl_type_targetsinfo.h"
#include "road_info_provider.h"
#include "road_model_subject.h"
#include "ros_recv.h"
#include "task.h"
#include "vehicle_state_provider.h"
#include "straight_driving_detector.h"
#include "boundary_builder.h"
#include "ats_switch_decider.h"
//using namespace cem_interfaces;
namespace rm {

/**
 * @class RoadModel
 * @brief RM的主类 主要包括接收local_view信息 并且处理成发送给BA的RoadInfo
 */
class RoadModel {
 public:
  RoadModel() = default;

  explicit RoadModel(const std::shared_ptr<DependencyInjector> &injector)
      : injector_(injector) {}

  ~RoadModel();

  /**
   * @brief RM name.
   */
  std::string Name() const;

  common::Status ObserverInit();
  /**
   * @brief 使用配置量初始化模型
   * @return initialization status
   */
  common::Status Init();
  LocalView local_view() const { return local_view_; }
  /**
   * @brief main logic of the rm, runs periodically triggered by
   * timer.
   * @return ptr_roadinfo 返回要发给BA的环境模型信息
   */
  void RunOnce(const LocalView &local_view,
               dpc::RoadInfo *const ptr_roadinfo,
               dpc::RMSMMessage *const ptr_rm_sm_msg,
               dpc::TargetsInfo *const ptr_targetsinfo);

  RoadInfoProvider *mutable_road_info_provider();

  Frame *mutable_frame();

 private:
  /**
   * @brief 按照task顺序处理每条corridor
   * @param corridor_info 需要处理的corridor信息
   * @return
   */
  common::Status Process();

  common::Status PreProcess();

  common::Status PostProcess();

  common::Status ProcessOnLane(LaneInfo *lane_info);

  void RelayCemInputStatus(dpc::RMSMMessage *const ptr_rm_sm_msg);

  void ResetInjector();

//   uint16_t RunCounter();

  common::Status UpdateOutputInfo(
      dpc::RoadInfo *ptr_roadinfo,
      dpc::RMSMMessage *ptr_rm_sm_msg,
      dpc::TargetsInfo *ptr_targetsinfo);

  /**
   * @brief 初始化框架
   * @return 初始化状态
   */
  common::Status InitFrame(const uint32_t sequence_num);

 private:
  size_t seq_num_ = 0;
//   uint16_t counters_;
  std::map<DynamicConfig::ConfigType, std::unique_ptr<Task>>
      tasks_;  //存储已经创建过的task

  std::vector<Task *> task_list_;  // rm的任务task调用列表 均指向堆区
  LocalView local_view_;           //输入数据

  std::shared_ptr<Subject> config_;
  std::map<DynamicConfig::ConfigType, std::shared_ptr<Observer>> observer_;
  // std::vector<std::shared_ptr<Observer>> observer_ ;
  std::unique_ptr<Frame> frame_;  //存储所有输入和输出信息
  std::shared_ptr<DependencyInjector> injector_;  //存储历史信息
  std::unique_ptr<RoadInfoProvider> road_info_provider_;
  std::array<bool, 9> tgt_existence_array_ = {
      false, false, false, false, false,
      false, false, false, false};  // flag indicating if the target has been
                                    // updated
  rm::Delay<uint8_t> line_crossed_delay_ = {0.0, 0.2, 0};
  rm::StraightDrivingDetector straight_driving_detector_;
  std::vector<rm::LanePosition> position_data_ = {
      LanePosition::LEFT6,    LanePosition::LEFT5,  LanePosition::LEFT4,
      LanePosition::LEFT3,    LanePosition::LEFT2,  LanePosition::LEFT1,
      LanePosition::EGO_LANE, LanePosition::RIGHT1, LanePosition::RIGHT2,
      LanePosition::RIGHT3,   LanePosition::RIGHT4, LanePosition::RIGHT5,
      LanePosition::RIGHT6};
  rm::ATSSwitchDecider ats_switch_decider_;
  // std::unique_ptr<VehicleStateProvider> vehicle_state_provider_;
};

}  // namespace rm
